@@ -2,7 +2,8 @@
 #%stage: block
 #%depends: dm
 #%provides: dmroot
-#%programs: /sbin/dmraid
+#%programs: dmraid
+#%programs: mkdir
 #%if: -n "$root_dmraid"
 #
 ##### Device Mapper Raid
@@ -17,6 +18,6 @@
 ## 
 
 [ -d /run/lock ] || mkdir -p /run/lock >/dev/null 2>&1
-/sbin/dmraid -a y -p
+dmraid -a y -p
 wait_for_events
 
