@@ -4,11 +4,11 @@
 #%provides: dmroot
 #%programs: /sbin/multipath /lib64/multipath/*
 #%if: "$root_mpath"
-#%modules: dm-multipath dm-round-robin scsi-dh-emc scsi-dh-hp_sw scsi-dh-rdac scsi-dh-alua
+#%modules: dm-multipath dm-round-robin dm-queue-length dm-least-pending dm-service-time scsi-dh-emc scsi-dh-hp_sw scsi-dh-rdac scsi-dh-alua
 #
 ##### Multipath
 ##
-## If the root device can be accessed using multiple device paths, 
+## If the root device can be accessed using multiple device paths,
 ## this initializes and waits for them
 ##
 ## Command line parameters
@@ -16,7 +16,7 @@
 ##
 ## root_mpath=1		use multipath
 ## mpath_status=off	do not use multipath
-## 
+##
 
 load_modules
 
@@ -35,8 +35,7 @@ if [ "$mpath_status" != "off" ] ; then
   root_major=$(sed -n 's/\(.*\) device-mapper/\1/p' /proc/devices)
   # Rescan for multipath
   echo -n "Setup multipath devices: "
-  /sbin/multipath -v0
+  /sbin/multipath -v0 -B
   wait_for_events
   echo 'ok.'
 fi
-
