@@ -7,13 +7,13 @@
 update_blockdev
 
 # Check whether we are using LVM2 (only available when not using EVMS)
-if [ -z "$root_evms" ] && [ -x /sbin/lvdisplay ] ; then
+if [ -z "$root_evms" ] && [ -x "$(type -p lvdisplay)" ] ; then
   lvm_blockdev=
 
   for bd in $blockdev; do
     update_blockdev $bd
 	
-    vg_name=$(lvdisplay -c 2> /dev/null | sed -n "/.*:$blockmajor:$blockminor\$/p")
+    vg_name=$(lvdisplay -c 2> /dev/null | sed -n "/:${blockmajor}:${blockminor}$/p")
     vg_dev=${vg_name%%:*}
     vg_name=${vg_name#*:}
     vg_root=${vg_name%%:*}
@@ -34,16 +34,14 @@ fi
 
 if use_script lvm2; then
     tmp_root_dm=1 # lvm needs dm
-	mkdir -p $tmp_mnt/etc/sysconfig
     mkdir -p $tmp_mnt/etc/lvm
     mkdir -p $tmp_mnt/var/lock/lvm
     cp -a /etc/lvm/lvm.conf $tmp_mnt/etc/lvm/
-	cp -a /etc/sysconfig/lvm $tmp_mnt/etc/sysconfig/
 # udev rule for lvm2
-	mkdir -p $tmp_mnt/lib/udev/rules.d
-	cp /lib/udev/rules.d/11-dm-lvm.rules $tmp_mnt/lib/udev/rules.d/
-	if [ -e /lib/udev/rules.d/64-lvm2.rules ]; then
-		cp /lib/udev/rules.d/64-lvm2.rules $tmp_mnt/lib/udev/rules.d/
+	mkdir -p $tmp_mnt/usr/lib/udev/rules.d
+	cp /usr/lib/udev/rules.d/11-dm-lvm.rules $tmp_mnt/usr/lib/udev/rules.d/
+	if [ -e /usr/lib/udev/rules.d/64-lvm2.rules ]; then
+		cp /usr/lib/udev/rules.d/64-lvm2.rules $tmp_mnt/usr/lib/udev/rules.d/
 	fi
 fi
 
