@@ -51,6 +51,7 @@ use Bootloader::Core;
 our @ISA = ('Bootloader::Core');
 use Bootloader::Path;
 
+use POSIX;
 use Data::Dumper;
 
 #module interface
@@ -62,17 +63,86 @@ Creates an instance of the Bootloader::Core::GRUB2EFI class.
 
 =cut
 
-sub new {
-    my $self = shift;
-    my $old = shift;
+sub GrubCfgSections {
+    my ($parent, $cfg, $sect) = @_;
+    my @m = $cfg =~ /(submenu|menuentry) \s+ '([^']*)' (.*?) ( \{ (?: [^{}]* | (?4))* \} )/sxg;
 
-    my $loader = $self->SUPER::new ($old);
-    bless ($loader);
+    for (my $i = 0; $i <= $#m; $i += 4) {
 
-    $loader->l_milestone ("GRUB2EFI::new: Created GRUB2EFI instance");
-    return $loader;
+        my $type  = $m[$i];
+        my $title = $m[$i+1];
+        my $args  = $m[$i+2];
+        my $cfg2  = $m[$i+3];
+        my $name  = ($parent) ? "$parent>$title" : "$title";
+
+        if ($type eq "menuentry") {
+            my %sect_info;
+
+            $sect_info{"name"} = $name;
+            $sect_info{"menuentry"} = $name;
+            $sect_info{"type"} = "image";
+
+            if ($args =~ m/'gnulinux-[^\s]+-recovery-[^\s]+'/) {
+                $sect_info{"usage"} = "linux_failsafe";
+            } else {
+                $sect_info{"usage"} = "linux";
+            }
+
+            # bnc#824609 - installation boot parameters not written to GRUB_CMDLINE_LINUX_DEFAULT
+            # we need to match linuxefi command, otherwise the kernel append will lost
+            if ($cfg2 =~ /^\s+(linux|linuxefi)\s+([^\s]+)\s*(.*)$/m) {
+                my $append = $3;
+                $sect_info{"image"} = $2;
+                my $command = $1;
+
+                if ($append =~ /root=/) {
+                    $append =~ s/root=([^\s]+)\s*//;
+                    $sect_info{"root"} = $1;
+                }
+
+                # the ro is part of linuxefi entries created by grub2 scripts
+                # not part of kernel append by yast
+                if ( $command eq "linuxefi" ) {
+                    $append =~ s/^ro\s*//;
+                }
+
+                if ($append =~ /vga=/) {
+                    $append =~ s/vga=([^\s]+)\s*//;
+                    $sect_info{"vgamode"} = $1;
+                }
+
+                $sect_info{"append"} = $append;
+            }
+            push @{$sect}, \%sect_info;
+        } elsif ($type eq "submenu") {
+            &GrubCfgSections ($name, $cfg2, $sect);
+        }
+    }
 }
 
+sub new
+{
+    my $self = shift;
+    my $ref = shift;
+    my $old = shift;
+
+    my $loader = $self->SUPER::new($ref, $old);
+    bless($loader);
+
+    my ($sysname, $nodename, $release, $version, $machine) = POSIX::uname();
+    # This exactly mimics grub-install logic and needs to be in sync with it
+    if ($machine =~ /^i.86.*/) {
+        $machine = "i386";
+    } elsif ($machine =~ /^(x86_64|amd64).*/) {
+        $machine = "x86_64";
+    }
+    my $target = "$machine-efi";
+    $loader->{'target'} = $target;
+
+    $loader->milestone("Created GRUB2EFI instance for target $target");
+
+    return $loader;
+}
 
 
 =item
@@ -88,8 +158,8 @@ sub ListFiles {
     my $self = shift;
     my @ret = (Bootloader::Path::Grub2_defaultconf());
 
-    if (-e Bootloader::Path::Grub2_eficonf()) {
-        push @ret, Bootloader::Path::Grub2_eficonf();
+    if (-e Bootloader::Path::Grub2_conf()) {
+        push @ret, Bootloader::Path::Grub2_conf();
     }
 
     return \@ret;
@@ -117,8 +187,7 @@ sub ParseLines {
 
     # and now proceed with /etc/default/grub
     my @defaultconf = @{$files{Bootloader::Path::Grub2_defaultconf()} || []};
-    $self->l_milestone ("GRUB2::Parselines: input from default conf :\n'" .
-                        join("'\n' ", @defaultconf) . "'");
+    $self->milestone("input from default conf :\n'" . join("'\n' ", @defaultconf) . "'");
 
     # prefix commented config with '@' instread of '#'
     # this is to new notation for "commented config" and
@@ -135,26 +204,14 @@ sub ParseLines {
         \@defaultconf
     );
 
-    my @confs = @{$files{Bootloader::Path::Grub2_eficonf()} || []};
-    my @entries = ();
-    my $submenu = "";
-    foreach my $conf (@confs) {
-        my $menuentry = "";
+    my @entries;
+    if (open (GRUBCFG, "<", Bootloader::Path::Grub2_conf())) {
+        local $/;
+        undef $/;
+        my $cfg = <GRUBCFG>;
 
-        if ($conf =~ /^menuentry\s+['"](.*?)['"]\s+/) {
-            $menuentry = $1;
-            $submenu = "";
-        } elsif ($conf =~ m/^submenu\s+['"](.*?)['"]\s+/) {
-            $submenu = $1;
-        }
-        if ($submenu ne "") {
-            if ($conf =~ m/^\s+menuentry\s+['"](.*?)['"]\s+/) {
-                $menuentry = "$submenu>$1"
-            }
-        }
-        if ($menuentry ne "") {
-	    push @entries, { "menuentry" =>  $menuentry };
-        }
+        &GrubCfgSections ("", $cfg, \@entries);
+        close (GRUBCFG);
     }
 
     $self->{"global"} = $glob_ref;
@@ -176,6 +233,7 @@ ParseLines on success, or undef on fail.
 sub CreateLines {
     my $self = shift;
     my $global = $self->{"global"};
+    my $sections = $self->{"sections"};
 
     if (defined $global->{"__lines"}) {
         foreach my $line (@{$global->{"__lines"}}) {
@@ -185,6 +243,26 @@ sub CreateLines {
         }
     }
 
+    foreach my $sect (@{$sections}) {
+
+        my $append = undef;
+
+        next unless $sect->{"__modified"} || 0;
+
+        if (exists $sect->{"usage"}) {
+            if ($sect->{"usage"} eq "linux") {
+                $append = \$global->{"append"};
+            } elsif ($sect->{"usage"} eq "linux_failsafe") {
+                $append = \$global->{"append_failsafe"};
+            }
+        }
+
+        if (defined $append && $sect->{"append"} ne ${$append}) {
+            ${$append} = $sect->{"append"};
+            last;
+        }
+    }
+
     my $grub2_defaultconf = $self->PrepareMenuFileLines (
         [],
         $global,
@@ -258,6 +336,10 @@ sub Global2Info {
             $ret{"distributor"} = $val;
         } elsif ($key =~ m/@?GRUB_CMDLINE_LINUX_RECOVERY$/) {
             $ret{"append_failsafe"} = $val;
+        } elsif ($key =~ m/@?GRUB_BACKGROUND/) {
+            $ret{"gfxbackground"} = $val;
+        } elsif ($key =~ m/@?GRUB_DISABLE_OS_PROBER$/) {
+            $ret{"os_prober"} = ($val eq "true") ? "false" : "true";
         }
     }
 
@@ -390,6 +472,14 @@ sub Info2Global {
                   '# Uncomment to get a beep at grub start'
                 ],
             },
+            {
+                'key' => 'GRUB_DISABLE_OS_PROBER',
+                'value' => 'false',
+                'comment_before' => [
+                  '# Skip 30_os-prober if you experienced very slow in probing them',
+                  '# WARNING foregin OS menu entries will be lost if set true here'
+                ],
+            },
         );
 
     }
@@ -399,14 +489,17 @@ sub Info2Global {
     # my $root = delete $globinfo{"root"} || "";
     my $vga = delete $globinfo{"vgamode"} || "";
     my $append = delete $globinfo{"append"} || "";
-    my $timeout = delete $globinfo{"timeout"} || "";
+    # YaST doesn't save bootloader settings when 'Timeout in seconds' is set to 0 (bnc#804176)
+    my $timeout = delete $globinfo{"timeout"} || 0;
     my $hiddenmenu = delete $globinfo{"hiddenmenu"} || "";
     my $terminal = delete $globinfo{"terminal"} || "";
     my $serial = delete $globinfo{"serial"} || "";
     my $gfxmode = delete $globinfo{"gfxmode"} || "";
     my $gfxtheme = delete $globinfo{"gfxtheme"} || "";
+    my $gfxbackground = delete $globinfo{"gfxbackground"} || "";
     my $distributor = delete $globinfo{"distributor"} || "";
     my $append_failsafe = delete $globinfo{"append_failsafe"} || "";
+    my $os_prober = delete $globinfo{"os_prober"} || "";
     # $root = " root=$root" if $root ne "";
     $vga = " vga=$vga" if $vga ne "";
     $append = " $append" if $append ne "";
@@ -418,6 +511,12 @@ sub Info2Global {
         $hidden_timeout = "0" if "$hidden_timeout" ne "";
     }
 
+    my $use_linuxefi = "";
+
+    if (defined $self->{secure_boot}) {
+        $use_linuxefi = $self->{secure_boot} ? "true" : "false";
+    }
+
     @lines = map {
         my $line_ref = $_;
         my $key = $line_ref->{"key"};
@@ -471,6 +570,21 @@ sub Info2Global {
         } elsif ($key =~ m/@?GRUB_CMDLINE_LINUX_RECOVERY$/) {
             $line_ref->{"value"} = "$append_failsafe" if "$append_failsafe" ne "";
             $append_failsafe = "";
+        } elsif ($key =~ m/@?GRUB_BACKGROUND/) {
+            if ($gfxbackground ne "") {
+                $line_ref->{"key"} = "GRUB_BACKGROUND";
+                $line_ref->{"value"} = $gfxbackground;
+            } else {
+                # delete the line once the value unset
+                $line_ref = undef;
+            }
+            $gfxbackground = "";
+        } elsif ($key =~ m/@?GRUB_DISABLE_OS_PROBER$/) {
+            $line_ref->{"value"} = ($os_prober eq "false") ? "true" : "false";
+            $os_prober = "";
+        } elsif ($key =~ m/@?GRUB_USE_LINUXEFI$/) {
+            $line_ref->{"value"} = "$use_linuxefi" if "$use_linuxefi" ne "";
+            $use_linuxefi = "";
         }
         defined $line_ref ? $line_ref : ();
     } @lines;
@@ -537,6 +651,28 @@ sub Info2Global {
             "value" => "$append_failsafe",
         }
     }
+
+    if ($gfxbackground ne "") {
+        push @lines, {
+            "key" => "GRUB_BACKGROUND",
+            "value" => "$gfxbackground",
+        }
+    }
+
+    if ("$os_prober" ne "") {
+        push @lines, {
+            "key" => "GRUB_DISABLE_OS_PROBER",
+            "value" => ($os_prober eq "false") ? "true" : "false",
+        }
+    }
+
+    if ("$use_linuxefi" ne "") {
+        push @lines, {
+            "key" => "GRUB_USE_LINUXEFI",
+            "value" => "$use_linuxefi",
+        }
+    }
+
     return \@lines;
 }
 
@@ -555,19 +691,20 @@ sub GetSettings {
 
     my $sections = $ret->{"sections"};
     my $globals = $ret->{"global"};
-    my $saved_entry = `/usr/bin/grub2-efi-editenv list|sed -n '/^saved_entry=/s/.*=//p'`;
+    my $saved_entry = `/usr/bin/grub2-editenv list|sed -n '/^saved_entry=/s/.*=//p'`;
 
     chomp $saved_entry;
-    if ($saved_entry ne "") {
-        if ($saved_entry =~ m/^\d+$/) {
-            my $sect = $sections->[$saved_entry];
+    if ($saved_entry eq "") {
+	$saved_entry = "0";
+    }
+    if ($saved_entry =~ m/^\d+$/) {
+	my $sect = $sections->[$saved_entry];
 
-            if (defined $sect) {
-                $globals->{"default"} = $sect->{"menuentry"};
-            }
-        } else {
-            $globals->{"default"} = $saved_entry;
-        }
+	if (defined $sect) {
+	    $globals->{"default"} = $sect->{"menuentry"};
+	}
+    } else {
+	$globals->{"default"} = $saved_entry;
     }
 
     return $ret;
@@ -587,15 +724,6 @@ sub SetSettings {
     my $self = shift;
     my %settings = %{+shift};
 
-    my $default  = delete $settings{"global"}->{"default"};
-
-    if (defined $default and $default ne "") {
-        $self->RunCommand (
-            "/usr/sbin/grub2-efi-set-default '$default'",
-            "/var/log/YaST2/y2log_bootloader"
-        );
-    }
-
     return $self->SUPER::SetSettings (\%settings);
 }
 
@@ -614,6 +742,7 @@ Returns undef on fail, defined nonzero value on success.
 sub UpdateBootloader {
     my $self = shift;
     my $avoid_init = shift;
+    my %glob = %{$self->{"global"}};
 
     # backup the config file
     # it really did nothing for now
@@ -626,9 +755,38 @@ sub UpdateBootloader {
         return $self->InitializeBootloader ();
     }
 
+    if (defined $self->{secure_boot})
+    {
+        my $opt = $self->{secure_boot} ? "--config-file=/boot/grub2/grub.cfg" : "--clean";
+        my $cmd = "/usr/sbin/shim-install";
+
+        if ($self->{secure_boot}) {
+            $ret = $self->RunCommand (
+                "$cmd $opt",
+                Bootloader::Path::BootCommandLogname()
+            );
+        } else {
+            $ret = $self->RunCommand (
+                "test -x $cmd && $cmd $opt || true",
+                Bootloader::Path::BootCommandLogname()
+            );
+		}
+
+        return 0 if (0 != $ret);
+    }
+
+    my $default  = delete $glob{"default"};
+
+    if (defined $default and $default ne "") {
+        $self->RunCommand (
+            "/usr/sbin/grub2-set-default '$default'",
+            Bootloader::Path::BootCommandLogname()
+        );
+    }
+
     return 0 == $self->RunCommand (
-        "/usr/sbin/grub2-efi-mkconfig -o /boot/grub2-efi/grub.cfg",
-        "/var/log/YaST2/y2log_bootloader"
+        "/usr/sbin/grub2-mkconfig -o /boot/grub2/grub.cfg",
+        Bootloader::Path::BootCommandLogname()
     );
 }
 
@@ -644,17 +802,47 @@ Returns undef on fail, defined nonzero value otherwise
 # boolean InitializeBootloader ()
 sub InitializeBootloader {
     my $self = shift;
+    my %glob = %{$self->{"global"}};
 
     my $ret = $self->RunCommand (
-        "/usr/sbin/grub2-efi-install",
-        "/var/log/YaST2/y2log_bootloader"
+        "/usr/sbin/grub2-install --target=$self->{'target'}",
+        Bootloader::Path::BootCommandLogname()
     );
 
     return 0 if (0 != $ret);
 
+    if (defined $self->{secure_boot})
+    {
+        my $opt = $self->{secure_boot} ? "--config-file=/boot/grub2/grub.cfg" : "--clean";
+        my $cmd = "/usr/sbin/shim-install";
+
+        if ($self->{secure_boot}) {
+            $ret = $self->RunCommand (
+                "$cmd $opt",
+                Bootloader::Path::BootCommandLogname()
+            );
+        } else {
+            $ret = $self->RunCommand (
+                "test -x $cmd && $cmd $opt || true",
+                Bootloader::Path::BootCommandLogname()
+            );
+		}
+
+        return 0 if (0 != $ret);
+    }
+
+    my $default  = delete $glob{"default"};
+
+    if (defined $default and $default ne "") {
+        $self->RunCommand (
+            "/usr/sbin/grub2-set-default '$default'",
+            Bootloader::Path::BootCommandLogname()
+        );
+    }
+
     return 0 == $self->RunCommand (
-        "/usr/sbin/grub2-efi-mkconfig -o /boot/grub2-efi/grub.cfg",
-        "/var/log/YaST2/y2log_bootloader"
+        "/usr/sbin/grub2-mkconfig -o /boot/grub2/grub.cfg",
+        Bootloader::Path::BootCommandLogname()
     );
 }
 
