@@ -354,7 +354,7 @@ sub GetHostsList {
 	$vhost_files = $data[0];
     }
 
-    foreach my $key (keys (%{$vhost_files})) {
+    foreach my $key (sort keys (%{$vhost_files})) {
 	given($key) {
 	    when ("ip-based") {
 		foreach my $hostList ($vhost_files->{'ip-based'}) {
@@ -421,7 +421,7 @@ sub GetHost {
 
     my $ret=undef;
 
-    foreach my $key (keys (%{$vhost_files})) {
+    foreach my $key (sort keys (%{$vhost_files})) {
 	given ($key) {
 	    when ("ip-based") {
 		foreach my $hostList ( $vhost_files->{'ip-based'} ) {
@@ -676,7 +676,7 @@ sub ModifyHost {
                 } 
 		elsif( $hostid ne 'default' and $tmp->{KEY} =~ /ServerTokens|TimeOut|ExtendedStatus/ ) {
                     # illegal keys in vhost
-                    return $self->SetError( summary => sprintf( __("Illegal key in vhost '%s'."),$tmp->{KEY}),
+                    return $self->SetError( summary => sprintf( __("Illegal key in virtual host '%s'."),$tmp->{KEY}),
                                             code    => "CHECK_PARAM_FAILED" );
                 } else {
                     push( @tmp, $tmp );
@@ -759,7 +759,7 @@ sub CreateHost {
         } 
 if( $key->{KEY} =~ /ServerTokens|TimeOut|ExtendedStatus/ ) {
             # illegal keys in vhost
-            return $self->SetError( summary => sprintf(__("Illegal key in vhost '%s'."), $key->{KEY}),
+            return $self->SetError( summary => sprintf(__("Illegal key in virtual host '%s'."), $key->{KEY}),
                                     code    => "CHECK_PARAM_FAILED" );
         } else {
             push( @tmp, $key );
